From 7fd33c77a138c32694721b35dc8bd66b5d6f8522 Mon Sep 17 00:00:00 2001
From: Tatsuo Ishii <ishii@postgresql.org>
Date: Mon, 6 Jan 2025 08:59:42 +0900
Subject: [PATCH v2] Add new StringInfo APIs to allow callers to specify buffer
 size.

Previously StringInfo APIs allocated buffers with fixed initial
allocation size of 1024 bytes. This may be too large and inappropriate
for some callers that can do with smaller memory buffers. To fix this,
introduce new APIs that allow callers to specify initial buffer size.

extern StringInfo makeStringInfoExt(int initsize);
extern void initStringInfoExt(StringInfo str, int initsize);

Existing APIs (makeStringInfo() and initStringInfo()) are changed to
call makeStringInfoExt and initStringInfoExt respectively with the
default buffer size of 1024.
---
 src/common/stringinfo.c      | 37 ++++++++++++++++++++++++++++++++----
 src/include/lib/stringinfo.h | 29 +++++++++++++++++++++++++++-
 2 files changed, 61 insertions(+), 5 deletions(-)

diff --git a/src/common/stringinfo.c b/src/common/stringinfo.c
index 55d2fbb864d..aa7e73604c8 100644
--- a/src/common/stringinfo.c
+++ b/src/common/stringinfo.c
@@ -36,12 +36,27 @@
  */
 StringInfo
 makeStringInfo(void)
+{
+    return makeStringInfoExt(STRINGINFO_DEFAULT_SIZE);
+}
+
+/*
+ * makeStringInfoExt(int initsize)
+ *
+ * Create an empty 'StringInfoData' & return a pointer to it.
+ * The initial memory allocation size is specified by 'initsize'.
+ * The valid range for 'initsize' is 1 to MaxAllocSize.
+ */
+StringInfo
+makeStringInfoExt(int initsize)
 {
 	StringInfo	res;
 
+	Assert(initsize > 0);
+
 	res = (StringInfo) palloc(sizeof(StringInfoData));
 
-	initStringInfo(res);
+	initStringInfoExt(res, initsize);
 
 	return res;
 }
@@ -55,10 +70,24 @@ makeStringInfo(void)
 void
 initStringInfo(StringInfo str)
 {
-	int			size = 1024;	/* initial default buffer size */
+    return initStringInfoExt(str, STRINGINFO_DEFAULT_SIZE);
+}
+
+/*
+ * initStringInfoExt
+ *
+ * Initialize a StringInfoData struct (with previously undefined contents)
+ * to describe an empty string.
+ * The initial memory allocation size is specified by 'initsize'.
+ * The valid range for 'initsize' is 1 to MaxAllocSize.
+ */
+void
+initStringInfoExt(StringInfo str, int initsize)
+{
+	Assert(initsize > 0);
 
-	str->data = (char *) palloc(size);
-	str->maxlen = size;
+	str->data = (char *) palloc(initsize);
+	str->maxlen = initsize;
 	resetStringInfo(str);
 }
 
diff --git a/src/include/lib/stringinfo.h b/src/include/lib/stringinfo.h
index 335208a9fda..c96df989bb0 100644
--- a/src/include/lib/stringinfo.h
+++ b/src/include/lib/stringinfo.h
@@ -55,11 +55,15 @@ typedef StringInfoData *StringInfo;
 
 
 /*------------------------
- * There are four ways to create a StringInfo object initially:
+ * There are six ways to create a StringInfo object initially:
  *
  * StringInfo stringptr = makeStringInfo();
  *		Both the StringInfoData and the data buffer are palloc'd.
  *
+ * StringInfo stringptr = makeStringInfoExt(initsize);
+ *		Same as makeStringInfo except the data buffer is allocated
+ *		with size 'initsize'.
+ *
  * StringInfoData string;
  * initStringInfo(&string);
  *		The data buffer is palloc'd but the StringInfoData is just local.
@@ -67,6 +71,11 @@ typedef StringInfoData *StringInfo;
  *		only live as long as the current routine.
  *
  * StringInfoData string;
+ * initStringInfoExt(&string, initsize);
+ *		Same as initStringInfo except the data buffer is allocated
+ *		with size 'initsize'.
+ *
+ * StringInfoData string;
  * initReadOnlyStringInfo(&string, existingbuf, len);
  *		The StringInfoData's data field is set to point directly to the
  *		existing buffer and the StringInfoData's len is set to the given len.
@@ -100,12 +109,22 @@ typedef StringInfoData *StringInfo;
  *-------------------------
  */
 
+#define STRINGINFO_DEFAULT_SIZE 1024	/* default initial allocation size */
+
 /*------------------------
  * makeStringInfo
  * Create an empty 'StringInfoData' & return a pointer to it.
  */
 extern StringInfo makeStringInfo(void);
 
+/*------------------------
+ * makeStringInfoExt
+ * Create an empty 'StringInfoData' & return a pointer to it.
+ * The data buffer is allocated with size 'initsize'.
+ * The valid range for 'initsize' is 1 to MaxAllocSize.
+ */
+extern StringInfo makeStringInfoExt(int initsize);
+
 /*------------------------
  * initStringInfo
  * Initialize a StringInfoData struct (with previously undefined contents)
@@ -113,6 +132,14 @@ extern StringInfo makeStringInfo(void);
  */
 extern void initStringInfo(StringInfo str);
 
+/*------------------------
+ * initStringInfoExt
+ * Initialize a StringInfoData struct (with previously undefined contents) to
+ * describe an empty string. The data buffer is allocated with size
+ * 'initsize'. The valid range for 'initsize' is 1 to MaxAllocSize.
+ */
+extern void initStringInfoExt(StringInfo str, int initsize);
+
 /*------------------------
  * initReadOnlyStringInfo
  * Initialize a StringInfoData struct from an existing string without copying
-- 
2.25.1

